@@ -48,18 +48,14 @@ Bmp8::~Bmp8() {
 // free buffer
 void Bmp8::FreeBmpBuffer(unsigned char **buff) {
   if (buff != NULL) {
-    if (buff[0] != NULL) {
-      delete []buff[0];
-    }
+    delete []buff[0];
     delete []buff;
   }
 }
 
 void Bmp8::FreeBmpBuffer(unsigned int **buff) {
   if (buff != NULL) {
-    if (buff[0] != NULL) {
-      delete []buff[0];
-    }
+    delete []buff[0];
     delete []buff;
   }
 }
@@ -76,17 +72,10 @@ unsigned char **Bmp8::CreateBmpBuffer(unsigned char init_val) {
   stride_ = ((wid_ % 4) == 0) ? wid_ : (4 * (1 + (wid_ / 4)));
 
   buff = (unsigned char **) new unsigned char *[hgt_ * sizeof(*buff)];
-  if (!buff) {
-    delete []buff;
-    return NULL;
-  }
 
   // alloc and init memory for buffer and line buffer
   buff[0] = (unsigned char *)
       new unsigned char[stride_ * hgt_ * sizeof(*buff[0])];
-  if (!buff[0]) {
-    return NULL;
-  }
 
   memset(buff[0], init_val, stride_ * hgt_ * sizeof(*buff[0]));
 
@@ -104,16 +93,9 @@ unsigned int ** Bmp8::CreateBmpBuffer(int wid, int hgt,
 
   // compute stride (align on 4 byte boundries)
   buff = (unsigned int **) new unsigned int *[hgt * sizeof(*buff)];
-  if (!buff) {
-    delete []buff;
-    return NULL;
-  }
 
   // alloc and init memory for buffer and line buffer
   buff[0] = (unsigned int *) new unsigned int[wid * hgt * sizeof(*buff[0])];
-  if (!buff[0]) {
-    return NULL;
-  }
 
   memset(buff[0], init_val, wid * hgt * sizeof(*buff[0]));
 
@@ -176,9 +158,6 @@ bool Bmp8::LoadFromCharDumpFile(CachedFile *fp) {
 
   // alloc memory & read the 3 channel buffer
   buff = new unsigned char[buf_size];
-  if (buff == NULL) {
-    return false;
-  }
 
   if (fp->Read(buff, buf_size) != buf_size) {
     delete []buff;
@@ -217,9 +196,6 @@ bool Bmp8::LoadFromCharDumpFile(CachedFile *fp) {
 Bmp8 * Bmp8::FromCharDumpFile(CachedFile *fp) {
   // create a Bmp8 object
   Bmp8 *bmp_obj = new Bmp8(0, 0);
-  if (bmp_obj == NULL) {
-    return NULL;
-  }
 
   if (bmp_obj->LoadFromCharDumpFile(fp) == false) {
     delete bmp_obj;
@@ -271,9 +247,6 @@ bool Bmp8::LoadFromCharDumpFile(FILE *fp) {
 
   // alloc memory & read the 3 channel buffer
   buff = new unsigned char[buf_size];
-  if (buff == NULL) {
-    return false;
-  }
 
   if (fread(buff, 1, buf_size, fp) != buf_size) {
     delete []buff;
@@ -312,9 +285,6 @@ bool Bmp8::LoadFromCharDumpFile(FILE *fp) {
 Bmp8 * Bmp8::FromCharDumpFile(FILE *fp) {
   // create a Bmp8 object
   Bmp8 *bmp_obj = new Bmp8(0, 0);
-  if (bmp_obj == NULL) {
-    return NULL;
-  }
 
   if (bmp_obj->LoadFromCharDumpFile(fp) == false) {
     delete bmp_obj;
@@ -549,9 +519,6 @@ bool Bmp8::SaveBmp2CharDumpFile(FILE *fp) const {
 
   // alloc memory & write the 3 channel buffer
   buff = new unsigned char[buf_size];
-  if (buff == NULL) {
-    return false;
-  }
 
   // copy the data
   for (y = 0, pix = 0; y < hgt_; y++) {
@@ -703,7 +670,7 @@ ConComp ** Bmp8::FindConComps(int *concomp_cnt, int min_size) const {
         // if there was no foreground pix, then create a new concomp
         if (master_concomp == NULL) {
           master_concomp = new ConComp();
-          if (master_concomp == NULL || master_concomp->Add(x, y) == false) {
+          if (master_concomp->Add(x, y) == false) {
             fprintf(stderr, "Cube ERROR (Bmp8::FindConComps): could not "
                     "allocate or add a connected component\n");
             FreeBmpBuffer(out_bmp_array);
@@ -715,13 +682,6 @@ ConComp ** Bmp8::FindConComps(int *concomp_cnt, int min_size) const {
           if ((alloc_concomp_cnt % kConCompAllocChunk) == 0) {
             ConComp **temp_con_comp =
                 new ConComp *[alloc_concomp_cnt + kConCompAllocChunk];
-            if (temp_con_comp == NULL) {
-              fprintf(stderr, "Cube ERROR (Bmp8::FindConComps): could not "
-                      "extend array of connected components\n");
-              FreeBmpBuffer(out_bmp_array);
-              delete []concomp_array;
-              return NULL;
-            }
 
             if (alloc_concomp_cnt > 0) {
               memcpy(temp_con_comp, concomp_array,
@@ -778,9 +738,6 @@ bool Bmp8::ComputeTanTable() {
   // alloc memory for tan table
   delete []tan_table_;
   tan_table_ = new float[kDeslantAngleCount];
-  if (tan_table_ == NULL) {
-    return false;
-  }
 
   for (ang_idx = 0, ang_val = kMinDeslantAngle;
        ang_idx < kDeslantAngleCount; ang_idx++) {
@@ -825,10 +782,6 @@ bool Bmp8::Deslant() {
   int **angle_hist = new int*[kDeslantAngleCount];
   for (ang_idx = 0; ang_idx < kDeslantAngleCount; ang_idx++) {
     angle_hist[ang_idx] = new int[des_wid];
-    if (angle_hist[ang_idx] == NULL) {
-      delete[] angle_hist;
-      return false;
-    }
     memset(angle_hist[ang_idx], 0, des_wid * sizeof(*angle_hist[ang_idx]));
   }
 
@@ -1010,10 +963,6 @@ bool Bmp8::HorizontalDeslant(double *deslant_angle) {
   int **angle_hist = new int*[kDeslantAngleCount];
   for (ang_idx = 0; ang_idx < kDeslantAngleCount; ang_idx++) {
     angle_hist[ang_idx] = new int[des_hgt];
-    if (angle_hist[ang_idx] == NULL) {
-      delete[] angle_hist;
-      return false;
-    }
     memset(angle_hist[ang_idx], 0, des_hgt * sizeof(*angle_hist[ang_idx]));
   }
 
@@ -1122,9 +1071,6 @@ float Bmp8::MeanHorizontalHistogramEntropy() const {
 
 int *Bmp8::HorizontalHistogram() const {
   int *hist = new int[hgt_];
-  if (hist == NULL) {
-    return NULL;
-  }
 
   // compute histograms
   for (int y = 0; y < hgt_; y++) {
