@@ -38,12 +38,7 @@
 #include "unicode/uchar.h"  // from libicu
 #include "util.h"
 
-#ifndef USE_STD_NAMESPACE
-// Compatability with pango 1.20.
-#include "pango/pango-glyph-item-private.h"
-#define pango_glyph_item_iter_init_start _pango_glyph_item_iter_init_start
-#define pango_glyph_item_iter_next_cluster _pango_glyph_item_iter_next_cluster
-#else
+#ifdef USE_STD_NAMESPACE
 using std::map;
 using std::max;
 using std::min;
@@ -57,7 +52,7 @@ static const int kDefaultOutputResolution = 300;
 // Word joiner (U+2060) inserted after letters in ngram mode, as per
 // recommendation in http://unicode.org/reports/tr14/ to avoid line-breaks at
 // hyphens and other non-alpha characters.
-static const char* kWordJoinerUTF8 = "\u2060";
+static const char* kWordJoinerUTF8 = "\xE2\x81\xA0";  // u8"\u2060";
 static const char32 kWordJoiner = 0x2060;
 
 static bool IsCombiner(int ch) {
@@ -72,6 +67,12 @@ static string EncodeAsUTF8(const char32 ch32) {
   return string(uni_ch.utf8(), uni_ch.utf8_len());
 }
 
+// Returns true with probability 'prob'.
+static bool RandBool(const double prob, TRand* rand) {
+  if (prob == 1.0) return true;
+  if (prob == 0.0) return false;
+  return rand->UnsignedRand(1.0) < prob;
+}
 
 /* static */
 Pix* CairoARGB32ToPixFormat(cairo_surface_t *surface) {
@@ -94,21 +95,37 @@ Pix* CairoARGB32ToPixFormat(cairo_surface_t *surface) {
 }
 
 StringRenderer::StringRenderer(const string& font_desc, int page_width,
-                               int page_height) :
-    page_width_(page_width), page_height_(page_height),
-    h_margin_(50), v_margin_(50), char_spacing_(0), leading_(0),
-    vertical_text_(false), gravity_hint_strong_(false),
-    render_fullwidth_latin_(false) ,drop_uncovered_chars_(true),
-    strip_unrenderable_words_(false), add_ligatures_(false),
-    output_word_boxes_(false), surface_(NULL), cr_(NULL),
-    layout_(NULL), start_box_(0), page_(0), box_padding_(0),
-    total_chars_(0), font_index_(0), last_offset_(0) {
-  pen_color_[0] = 0.0;
-  pen_color_[1] = 0.0;
-  pen_color_[2] = 0.0;
+                               int page_height)
+    : page_width_(page_width),
+      page_height_(page_height),
+      h_margin_(50),
+      v_margin_(50),
+      pen_color_{0.0, 0.0, 0.0},
+      char_spacing_(0),
+      leading_(0),
+      vertical_text_(false),
+      gravity_hint_strong_(false),
+      render_fullwidth_latin_(false),
+      underline_start_prob_(0),
+      underline_continuation_prob_(0),
+      underline_style_(PANGO_UNDERLINE_SINGLE),
+      features_(NULL),
+      drop_uncovered_chars_(true),
+      strip_unrenderable_words_(false),
+      add_ligatures_(false),
+      output_word_boxes_(false),
+      surface_(NULL),
+      cr_(NULL),
+      layout_(NULL),
+      start_box_(0),
+      page_(0),
+      box_padding_(0),
+      page_boxes_(NULL),
+      total_chars_(0),
+      font_index_(0),
+      last_offset_(0) {
   set_font(font_desc);
   set_resolution(kDefaultOutputResolution);
-  page_boxes_ = NULL;
 }
 
 bool StringRenderer::set_font(const string& desc) {
@@ -122,7 +139,16 @@ void StringRenderer::set_resolution(const int resolution) {
   font_.set_resolution(resolution);
 }
 
+void StringRenderer::set_underline_start_prob(const double frac) {
+  underline_start_prob_ = min(max(frac, 0.0), 1.0);
+}
+
+void StringRenderer::set_underline_continuation_prob(const double frac) {
+  underline_continuation_prob_ = min(max(frac, 0.0), 1.0);
+}
+
 StringRenderer::~StringRenderer() {
+  free(features_);
   ClearBoxes();
   FreePangoCairo();
 }
@@ -132,7 +158,10 @@ void StringRenderer::InitPangoCairo() {
   surface_ = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, page_width_,
                                         page_height_);
   cr_ = cairo_create(surface_);
-  layout_ = pango_cairo_create_layout(cr_);
+  {
+    DISABLE_HEAP_LEAK_CHECK;
+    layout_ = pango_cairo_create_layout(cr_);
+  }
 
   if (vertical_text_) {
     PangoContext* context = pango_layout_get_context(layout_);
@@ -175,6 +204,13 @@ void StringRenderer::SetLayoutProperties() {
     spacing_attr->end_index = static_cast<guint>(-1);
     pango_attr_list_change(attr_list, spacing_attr);
   }
+#if (PANGO_VERSION_MAJOR == 1 && PANGO_VERSION_MINOR >= 38)
+  if (add_ligatures_) {
+    set_features("liga, clig, dlig, hlig");
+    PangoAttribute* feature_attr = pango_attr_font_features_new(features_);
+    pango_attr_list_change(attr_list, feature_attr);
+  }
+#endif
   pango_layout_set_attributes(layout_, attr_list);
   pango_attr_list_unref(attr_list);
   // Adjust line spacing
@@ -198,6 +234,50 @@ void StringRenderer::FreePangoCairo() {
   }
 }
 
+void StringRenderer::SetWordUnderlineAttributes(const string& page_text) {
+  if (underline_start_prob_ == 0) return;
+  PangoAttrList* attr_list = pango_layout_get_attributes(layout_);
+
+  const char* text = page_text.c_str();
+  int offset = 0;
+  TRand rand;
+  bool started_underline = false;
+  PangoAttribute* und_attr = NULL;
+
+  while (offset < page_text.length()) {
+    offset += SpanUTF8Whitespace(text + offset);
+    if (offset == page_text.length()) break;
+
+    int word_start = offset;
+    int word_len = SpanUTF8NotWhitespace(text + offset);
+    offset += word_len;
+    if (started_underline) {
+      // Should we continue the underline to the next word?
+      if (RandBool(underline_continuation_prob_, &rand)) {
+        // Continue the current underline to this word.
+        und_attr->end_index = word_start + word_len;
+      } else {
+        // Otherwise end the current underline attribute at the end of the
+        // previous word.
+        pango_attr_list_insert(attr_list, und_attr);
+        started_underline = false;
+        und_attr = NULL;
+      }
+    }
+    if (!started_underline && RandBool(underline_start_prob_, &rand)) {
+      // Start a new underline attribute
+      und_attr = pango_attr_underline_new(underline_style_);
+      und_attr->start_index = word_start;
+      und_attr->end_index = word_start + word_len;
+      started_underline = true;
+    }
+  }
+  // Finish the current underline attribute at the end of the page.
+  if (started_underline) {
+    und_attr->end_index = page_text.length();
+    pango_attr_list_insert(attr_list, und_attr);
+  }
+}
 
 // Returns offset in utf8 bytes to first page.
 int StringRenderer::FindFirstPageBreakOffset(const char* text,
@@ -265,7 +345,13 @@ void StringRenderer::ClearBoxes() {
   boxaDestroy(&page_boxes_);
 }
 
-void StringRenderer::WriteAllBoxes(const string& filename) const {
+string StringRenderer::GetBoxesStr() {
+  BoxChar::PrepareToWrite(&boxchars_);
+  return BoxChar::GetTesseractBoxStr(page_height_, boxchars_);
+}
+
+void StringRenderer::WriteAllBoxes(const string& filename) {
+  BoxChar::PrepareToWrite(&boxchars_);
   BoxChar::WriteTesseractBoxFile(filename, page_height_, boxchars_);
 }
 
@@ -312,7 +398,7 @@ bool StringRenderer::GetClusterStrings(vector<string>* cluster_text) {
        it != start_byte_to_text.end(); ++it) {
     cluster_text->push_back(it->second);
   }
-  return cluster_text->size();
+  return !cluster_text->empty();
 }
 
 // Merges an array of BoxChars into words based on the identification of
@@ -412,7 +498,7 @@ void StringRenderer::ComputeClusterBoxes() {
     const int end_byte_index = cluster_start_to_end_index[start_byte_index];
     string cluster_text = string(text + start_byte_index,
                                  end_byte_index - start_byte_index);
-    if (cluster_text.size() && cluster_text[0] == '\n') {
+    if (!cluster_text.empty() && cluster_text[0] == '\n') {
       tlog(2, "Skipping newlines at start of text.\n");
       continue;
     }
@@ -512,11 +598,12 @@ void StringRenderer::ComputeClusterBoxes() {
       all_boxes = boxaCreate(0);
     boxaAddBox(all_boxes, page_boxchars[i]->mutable_box(), L_CLONE);
   }
-  boxaGetExtent(all_boxes, NULL, NULL, &page_box);
-  boxaDestroy(&all_boxes);
-  if (page_boxes_ == NULL)
-    page_boxes_ = boxaCreate(0);
-  boxaAddBox(page_boxes_, page_box, L_INSERT);
+  if (all_boxes != NULL) {
+    boxaGetExtent(all_boxes, NULL, NULL, &page_box);
+    boxaDestroy(&all_boxes);
+    if (page_boxes_ == NULL) page_boxes_ = boxaCreate(0);
+    boxaAddBox(page_boxes_, page_box, L_INSERT);
+  }
 }
 
 
@@ -559,6 +646,17 @@ int StringRenderer::StripUnrenderableWords(string* utf8_text) const {
   return num_dropped;
 }
 
+int StringRenderer::RenderToGrayscaleImage(const char* text, int text_length,
+                                           Pix** pix) {
+  Pix *orig_pix = NULL;
+  int offset = RenderToImage(text, text_length, &orig_pix);
+  if (orig_pix) {
+    *pix = pixConvertTo8(orig_pix, false);
+    pixDestroy(&orig_pix);
+  }
+  return offset;
+}
+
 int StringRenderer::RenderToBinaryImage(const char* text, int text_length,
                                         int threshold, Pix** pix) {
   Pix *orig_pix = NULL;
@@ -688,6 +786,9 @@ int StringRenderer::RenderToImage(const char* text, int text_length,
     // Add ligatures wherever possible, including custom ligatures.
     page_text = LigatureTable::Get()->AddLigatures(page_text, &font_);
   }
+  if (underline_start_prob_ > 0) {
+    SetWordUnderlineAttributes(page_text);
+  }
 
   pango_layout_set_text(layout_, page_text.c_str(), page_text.length());
 
@@ -702,8 +803,11 @@ int StringRenderer::RenderToImage(const char* text, int text_length,
     // If the target surface or transformation properties of the cairo instance
     // have changed, update the pango layout to reflect this
     pango_cairo_update_layout(cr_, layout_);
-    // Draw the pango layout onto the cairo surface
-    pango_cairo_show_layout(cr_, layout_);
+    {
+      DISABLE_HEAP_LEAK_CHECK;  // for Fontconfig
+      // Draw the pango layout onto the cairo surface
+      pango_cairo_show_layout(cr_, layout_);
+    }
     *pix = CairoARGB32ToPixFormat(surface_);
   }
   ComputeClusterBoxes();
@@ -717,7 +821,7 @@ int StringRenderer::RenderToImage(const char* text, int text_length,
 // RenderString, except that it ignores the font set at construction and works
 // through all the fonts, returning 0 until they are exhausted, at which point
 // it returns the value it should have returned all along, but no pix this time.
-// Fonts that don't contain a large proportion of the characters in the string
+// Fonts that don't contain a given proportion of the characters in the string
 // get skipped.
 // Fonts that work each get rendered and the font name gets added
 // to the image.
@@ -728,13 +832,15 @@ int StringRenderer::RenderToImage(const char* text, int text_length,
 // int offset = 0;
 // do {
 //   Pix *pix;
-//   offset += renderer.RenderAllFontsToImage(txt + offset,
+//   offset += renderer.RenderAllFontsToImage(min_proportion, txt + offset,
 //                                            strlen(txt + offset), NULL, &pix);
 //   ...
 // } while (offset < strlen(text));
 //
-int StringRenderer::RenderAllFontsToImage(const char* text, int text_length,
+int StringRenderer::RenderAllFontsToImage(double min_coverage,
+                                          const char* text, int text_length,
                                           string* font_used, Pix** image) {
+  *image = NULL;
   // Select a suitable font to render the title with.
   const char kTitleTemplate[] = "%s : %d hits = %.2f%%, raw = %d = %.2f%%";
   string title_font;
@@ -764,7 +870,7 @@ int StringRenderer::RenderAllFontsToImage(const char* text, int text_length,
     int raw_score = 0;
     int ok_chars = FontUtils::FontScore(char_map_, all_fonts[i], &raw_score,
                                         NULL);
-    if (ok_chars > 0 && ok_chars == total_chars_) {
+    if (ok_chars > 0 && ok_chars >= total_chars_ * min_coverage) {
       set_font(all_fonts[i]);
       int offset = RenderToBinaryImage(text, text_length, 128, image);
       ClearBoxes();  // Get rid of them as they are garbage.
@@ -798,10 +904,9 @@ int StringRenderer::RenderAllFontsToImage(const char* text, int text_length,
               all_fonts[i].c_str(), ok_chars, 100.0 * ok_chars / total_chars_);
     }
   }
-  *image = NULL;
   font_index_ = 0;
   char_map_.clear();
-  return last_offset_;
+  return last_offset_ == 0 ? -1 : last_offset_;
 }
 
 }  // namespace tesseract
