@@ -48,8 +48,8 @@ typedef unsigned char LanguageModelFlagsType;
 /// Each ViterbiStateEntry contains information from various components of the
 /// language model: dawgs in which the path is found, character ngram model
 /// probability of the path, script/chartype/font consistency info, state for
-/// language-specific heuristics (e.g. hyphenated and compound words, lower/upper
-/// case preferences, etc).
+/// language-specific heuristics (e.g. hyphenated and compound words,
+/// lower/upper case preferences, etc).
 ///
 /// Each ViterbiStateEntry also contains the parent pointer, so that the path
 /// that it represents (WERD_CHOICE) can be constructed by following these
@@ -59,13 +59,9 @@ typedef unsigned char LanguageModelFlagsType;
 /// component. It stores the set of active dawgs in which the sequence of
 /// letters on a path can be found.
 struct LanguageModelDawgInfo {
-  LanguageModelDawgInfo(DawgPositionVector *a, PermuterType pt) : permuter(pt) {
-    active_dawgs = new DawgPositionVector(*a);
-  }
-  ~LanguageModelDawgInfo() {
-    delete active_dawgs;
-  }
-  DawgPositionVector *active_dawgs;
+  LanguageModelDawgInfo(const DawgPositionVector *a, PermuterType pt)
+    : active_dawgs(*a), permuter(pt) {}
+  DawgPositionVector active_dawgs;
   PermuterType permuter;
 };
 
@@ -165,13 +161,13 @@ struct ViterbiStateEntry : public ELIST_LINK {
 
   /// Various information about the characters on the path represented
   /// by this ViterbiStateEntry.
-  float ratings_sum;  //< sum of ratings of character on the path
-  float min_certainty;  //< minimum certainty on the path
-  int adapted;  //< number of BLOB_CHOICES from adapted templates
-  int length;  //< number of characters on the path
+  float ratings_sum;     //< sum of ratings of character on the path
+  float min_certainty;   //< minimum certainty on the path
+  int adapted;           //< number of BLOB_CHOICES from adapted templates
+  int length;            //< number of characters on the path
   float outline_length;  //< length of the outline so far
   LMConsistencyInfo consistency_info;  //< path consistency info
-  AssociateStats associate_stats;  //< character widths/gaps/seams
+  AssociateStats associate_stats;      //< character widths/gaps/seams
 
   /// Flags for marking the entry as a top choice path with
   /// the smallest rating or lower/upper case letters).
